/*
 * NAME:
 *	sslc - routines useful for SSL clients
 *
 * SYNOPSIS:
 *	const char *setMyId(const char *"myname")
 *	int	sslc_init(const char *"cert_file",
 *			const char *"key_file",
 *			const char *"cipher_list",
 *			int "verify_flag", int "flags")
 *	int	sslc_start_cb(int "fd", int "active",
 *			int "flags", int (*"cb")(), int "vf")
 *	int	sslc_start(int "fd", int "active", int "flags")
 *	
 * DESCRIPTION:
 *	These routines are intended for use by SSL based clients.
 *	Routines which are shared between clients and servers are
 *	documented in sslfd(3).
 *
 *	setMyId() should be called before anything else, as it sets
 *	the prompt that SSLeay will use to prompt for passwords.  The
 *	aim is that the client will tell the user what its name and
 *	process id are so that the user knows to ignore any important
 *	message that does not include that tag - such as a prompt for
 *	their certificate password.
 *	
 *	A typical client then calls:
 *
 *	sslc_init("cert_file", "key_file", "cipher_list",
 *		"verify_flag", "init_flags");
 *
 *	which calls
 *
 *	ssl_init("cert_file", "key_file", "cipher_list",
 *		"verify_flag", "verify_cb", "log_lev", "init_flags"): 
 *
 *	and provides appropriate callback and "log_lev" arguments.
 *	The client can then call one of:
 *.nf
 *
 *	sslc_start("sock", "active", "flags")
 *	sslc_start_cb("sock", "active", "flags", "verify_cb",
 *		"verify_flag")
 *	ssl_start_cb("sock", "log_lev", "active", "verify_cb",
 *		"verify_flag")
 *.fi
 *
 *	The only feature that the sslc_start() routines have over
 *	ssl_start_cb() is that if bit one of "flags" is set, they
 *	issue a status message indicating that SSL is established and
 *	which cipher is being used.
 *
 * SEE ALSO:
 *	ssld(3), ssl_rcmd(3), ssl(3), netio(3).
 *
 * NOTES:
 *	The current version of SSLeay can be obtained from:
 *	'ftp://ftp.psy.uq.oz.au/pub/Crypto/SSL/SSLeay-x.xx.tar.gz'
 *
 *	The SSLeay FAQ by Tim Hudson <tjh@mincom.oz.au> can be found
 *	at 'http://www.psy.uq.oz.au/~ftp/Crypto'
 *	
 * AUTHOR:
 *	Simon J. Gerraty <sjg@quick.com.au>
 */
/*
 *	@(#)Copyright (c) 1996 Simon J. Gerraty.
 *	
 *	This is free software.	It comes with NO WARRANTY.
 *	Permission to use, modify and distribute this source code 
 *	is granted subject to the following conditions.
 *	1/ that the above copyright notice and this notice 
 *	are preserved in all copies and that due credit be given 
 *	to the author.	
 *	2/ that any changes to this code are clearly commented 
 *	as such so that the author does not get blamed for bugs 
 *	other than his own.
 *	
 *	Please send copies of changes and bug-fixes to:
 *	sjg@quick.com.au
 */
#ifndef lint
static char RCSid[] = "$Id: sslc.c,v 1.19 1998/11/12 04:01:10 sjg Exp $";
#endif

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <syslog.h>
#include <string.h>


#include "x509.h"
#include "ssl.h"

#include "sslfd.h"

static int ssl_verify_verbose = 0;
extern char *MyId;

/*
 * <sjg> a simplified cb, so we can print useful errors
 */
static int
#ifdef SSLEAY8
ssl_verify_cb(ok, ctx)
	int ok;
	X509_STORE_CTX *ctx;
{
	const char *xs;
	int depth, error;
	char *name;

	xs = (char *) X509_STORE_CTX_get_current_cert(ctx);
	error = ctx->error;
	depth = ctx->error_depth;
#else
ssl_verify_cb(ok, xs, xi, depth, error)
	int ok;
	const char *xs, *xi;
	int depth, error;
{
	SSL_CTX *ssl_ctx, *sslfd_ctx();
	CERTIFICATE_CTX *ctx = 0;
	char *name;

	if ((ssl_ctx = sslfd_ctx()) != 0)
		ctx = ssl_ctx->cert;
#endif
	
	name = X509_NAME_ONELINE(X509_get_subject_name((X509 *)xs));
	
	if (!ok) {
		char *err = X509_cert_verify_error_string(error);
		
		if ((ssl_verify_flag & SSL_VERIFY_FAIL_IF_NO_PEER_CERT) == 0)
			ok=1;
		
		fprintf(stderr, "%s: %s: SSL cert verify: depth=%d,err=%s,xs=%s\n",
			MyId, (ok) ? "WARNING" : "ERROR", depth, err, name);
	}
	if (ok) {
		char *issuer = X509_NAME_ONELINE(X509_get_issuer_name((X509 *)xs));
		ASN1_UTCTIME *utc = X509_get_notAfter((X509 *)xs);
		
		if (utcSecsCmp(utc, 0, 30*24*3600) < 0) {
			fprintf(stderr,
				"%s: WARNING: expires: %s%.6s: cert=%s\n",
				MyId,
				(utc->data[0] == '9') ? "19" : "20",
				(char *) utc->data, name);
		}
		
		switch(check_crl(ctx, xs)) {
		case 0:				/* no CRL, guess they don't care */
#ifdef CRL_PEDANTIC
			if (ssl_verify_verbose)	
				fprintf(stderr,"%s: no CRL: depth=%d,xi=%s\n",
					MyId, depth, issuer);
#endif
			break;
		case -1:			/* expired CRL */
			if (ssl_verify_verbose)	
				fprintf(stderr,"%s: expired CRL: depth=%d,xi=%s\n",
					MyId, depth, issuer);
			break;
		case 1:				/* OK */
			break;
		default:			/* revoked */
			fprintf(stderr,"%s: ERROR cert REVOKED: depth=%d,xi=%s,xs=%s\n",
				MyId, depth, issuer, name);
			if (ssl_verify_flag & SSL_VERIFY_FAIL_IF_NO_PEER_CERT)
				ok=0;
			break;
		}
		free(issuer);
		if (ssl_verify_verbose > 0 && ok > 0) {
			fprintf(stderr, "%s: SSL cert verify: depth=%d,xs=%s\n",
				MyId, depth, name);
		}
	}
	free(name);
	return ok;
}

const char *
setMyId(myname)
	const char *myname;
{
	static char myid[64];
	char prompt[80];
	register const char *rcp;

	if (rcp = strrchr(myname, '/'))
		rcp++;
	else
		rcp = myname;
	sprintf(myid, "%.*s[%d]", sizeof(myid) - 10, rcp, getpid());
	MyId = myid;
#if SSLEAY_VERSION_NUMBER >= 0x0600
	sprintf(prompt, "%s: Enter PEM pass phrase:", MyId);
	EVP_set_pw_prompt(prompt);
#endif
	return rcp;
}

int
sslc_init(cert_file, key_file, cipher_list, verify_flag, flags)
	const char *cert_file, *key_file, *cipher_list;
	int verify_flag;
	int flags;
{
	ssl_verify_verbose = (flags & 1);
	
	return ssl_init(cert_file, key_file, cipher_list, verify_flag,
			ssl_verify_cb, -1, flags);
}

int
sslc_start_cb(fd, active, flags, cb, vf)
	int fd;
	int active;
	int flags;
	int (*cb) ();
	int vf;
{
	int sts = ssl_start_cb(fd, -1, active, cb, vf);

	if (sts > 0 && (flags & 1)) {
		(void) fprintf(stderr,
			       "%s: SSL Connected - Cipher %s\n",
			       MyId, SSL_get_cipher(ssl_getcon(fd)));
	}
	return sts;
}

int
sslc_start(fd, active, flags)
	int fd;
	int active;
	int flags;
{
	return sslc_start_cb(fd, active, flags, 0, -1);
}

